#!/usr/bin/env python

import errno
import logging
import os
import sys
import xmlrpc.client

from fuse import FUSE, FuseOSError, Operations

SERVER_URL = os.environ.get('SERVER_URL', 'localhost:5000')
LOGLEVEL = getattr(logging, os.environ.get('LOGLEVEL', 'ERROR'))
PROTOCOL = os.environ.get('PROTOCOL', 'http://')

logging.basicConfig(
    format='%(asctime)s [%(process)s] %(levelname)s %(pathname)s:%(lineno)s: %(message)s',  # NOQA E501
    # filename=basename_executable + '.log',
    # level=logging.INFO,
    level=LOGLEVEL,
    stream=sys.stderr,
)
logger = logging.getLogger(__name__)


def unwrap_exceptions_generator(func):
    def wrapper_generator(self, *args, **kwargs):
        data = self._call(func, *args, **kwargs)
        for i in data['data']:
            yield i
    return wrapper_generator

def unwrap_exceptions(func):
    def wrapper(self, *args, **kwargs):
        data = self._call(func, *args, **kwargs)
        return data['data']
    return wrapper

class Passthrough(Operations):

    def __init__(self, server_name, server_port):
        logger.info(f"Opening {server_name}:{server_port} for a GoFUSE mount!")
        self.server = xmlrpc.client.ServerProxy(
            f'{PROTOCOL}{server_name}:{server_port}/',
            allow_none=True,
        )
        logger.debug(self.server.system.listMethods())

    # Helper
    def _unpack(self, data):
        if isinstance(data['data'], xmlrpc.client.Binary):
            data['data'] = data['data'].data
        return data
    
    def _call(self, func, *args, **kwargs):
        logger.debug(f"Calling:{repr(func)} args:{repr(args)} kwargs:{repr(kwargs)})")
        callable = getattr(self.server, func.__name__)
        data = callable(*args, **kwargs)
        data = self._unpack(data)
        if data['exception'] is not None:
            exec(f"raise {data['exception']}")
        logger.debug(f"Data type is {type(data['data'])}")
        return data

    # Filesystem methods
    # ==================

    @unwrap_exceptions
    def access(self, *args, **kwargs):
        pass

    @unwrap_exceptions
    def chmod(self, *args, **kwargs):
        pass

    @unwrap_exceptions
    def chown(self, *args, **kwargs):
        pass

    @unwrap_exceptions
    def getattr(self, *args, **kwargs):
        pass

    @unwrap_exceptions
    def getxattr(self, *args, **kwargs):
        pass

    # Generator
    @unwrap_exceptions_generator
    def readdir(self, *args, **kwargs):
        pass

    @unwrap_exceptions
    def readlink(self, *args, **kwargs):
        pass

    @unwrap_exceptions
    def mknod(self, *args, **kwargs):
        pass

    @unwrap_exceptions
    def rmdir(self, *args, **kwargs):
        pass

    @unwrap_exceptions
    def mkdir(self, *args, **kwargs):
        pass

    @unwrap_exceptions
    def statfs(self, *args, **kwargs):
        pass

    @unwrap_exceptions
    def unlink(self, *args, **kwargs):
        pass

    @unwrap_exceptions
    def symlink(self, *args, **kwargs):
        pass

    @unwrap_exceptions
    def rename(self, *args, **kwargs):
        pass

    @unwrap_exceptions
    def link(self, *args, **kwargs):
        pass

    @unwrap_exceptions
    def utimens(self, *args, **kwargs):
        pass

    # File methods
    # ============

    @unwrap_exceptions
    def open(self, *args, **kwargs):
        pass

    @unwrap_exceptions
    def create(self, *args, **kwargs):
        pass

    @unwrap_exceptions
    def read(self, *args, **kwargs):
        pass

    @unwrap_exceptions
    def write(self, *args, **kwargs):
        pass

    @unwrap_exceptions
    def truncate(self, *args, **kwargs):
        pass

    @unwrap_exceptions
    def flush(self, *args, **kwargs):
        pass

    @unwrap_exceptions
    def release(self, *args, **kwargs):
        pass

    @unwrap_exceptions
    def fsync(self, *args, **kwargs):
        pass


def main():

    # Get configuration
    server_name, server_port = SERVER_URL.split(':')
    server_port = int(server_port)
    mountpoint = sys.argv[1]

    # Create client
    logger.info(f"Starting client fo {server_name}:{server_port}")
    logger.debug(f"Mounting '{mountpoint}'")
    FUSE(Passthrough(server_name, server_port), mountpoint, nothreads=True, foreground=True)

if __name__ == '__main__':
    main()



import os
import sys

from setuptools import setup

CURRENT_PYTHON = sys.version_info[:2]
REQUIRED_PYTHON = (3, 0)

# This check and everything above must remain compatible with Python 2.7.
if CURRENT_PYTHON < REQUIRED_PYTHON:
    sys.stderr.write(f"""
==========================
Unsupported Python version
==========================
GoFUSE was written for requires Python {REQUIRED_PYTHON}, but you're trying to install it on Python {CURRENT_PYTHON}. This may also be because you are using a version of pip that doesn't understand the python_requires classifier. Make sure you have pip >= 9.0 and setuptools >= 24.2, then try again:
    $ python -m pip install --upgrade pip setuptools
    $ python -m pip install GoFUSE
""")
    sys.exit(1)

from distutils.core import setup


setup(
    name='GoFUSE',
    version='0.0.7',
    author='Georg Gogo. BERNHARD',
    author_email='gogo@bluedynamics.com',
    packages=['GoFUSE',],
    url='https://pypi.org/project/GoFUSE/0.0.1/',
    license='GPL',
    scripts=[],
    description='GoFUSE is a pyfuse / xmlrpc based distributed filesystem.',
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    install_requires=[
        "fusepy >= 3.0.1",
    ],
)



#!/usr/bin/env python3

import errno
import logging
import os
import sys
from xmlrpc.server import SimpleXMLRPCRequestHandler, SimpleXMLRPCServer

from fuse import FUSE, FuseOSError, Operations

SERVER_URL = os.environ.get('SERVER_URL', '0.0.0.0:5000')
LOGLEVEL = getattr(logging, os.environ.get('LOGLEVEL', 'ERROR'))

logging.basicConfig(
    format='%(asctime)s [%(process)s] %(levelname)s %(pathname)s:%(lineno)s: %(message)s',  # NOQA E501
    # filename=basename_executable + '.log',
    # level=logging.INFO,
    level=LOGLEVEL,
    stream=sys.stderr,
)
logger = logging.getLogger(__name__)


def wrap_exeptions(func):

    def wrapper(*args, **kwargs):
        logger.debug(f"Calling:{repr(func)} args:{repr(args)} kwargs:{repr(kwargs)})")
        data = None
        exception = None
        try:
            data = func(*args, **kwargs)
        except Exception as e:
            logger.error(e)
            exception = repr(e)
        logger.debug(f"data:{repr(data)}")
        logger.debug(f"exception:{repr(exception)}")
        return {
            'data': data,
            'exception': exception,
        }

    return wrapper

class Passthrough(Operations):
    def __init__(self, root):
        self.root = root

    # Helpers
    # =======

    def _full_path(self, partial):
        partial = partial.lstrip("/")
        path = os.path.join(self.root, partial)
        return path

    # Filesystem methods
    # ==================

    @wrap_exeptions
    def access(self, path, mode):
        full_path = self._full_path(path)
        if not os.access(full_path, mode):
            raise FuseOSError(errno.EACCES)

    @wrap_exeptions
    def chmod(self, path, mode):
        full_path = self._full_path(path)
        return os.chmod(full_path, mode)

    @wrap_exeptions
    def chown(self, path, uid, gid):
        full_path = self._full_path(path)
        return os.chown(full_path, uid, gid)

    @wrap_exeptions
    def getattr(self, path, fh=None):
        full_path = self._full_path(path)
        st = os.lstat(full_path)
        data = dict((key, getattr(st, key)) for key in ('st_atime', 'st_ctime',
                    'st_gid', 'st_mode', 'st_mtime', 'st_nlink', 'st_size', 'st_uid'))
        return data

    @wrap_exeptions
    def getxattr(self, path, attribute, follow_symlinks=True):
        full_path = self._full_path(path)
        data = os.getxattr(full_path, attribute, follow_symlinks=follow_symlinks)
        return data

    @wrap_exeptions
    def readdir(self, path, fh):
        full_path = self._full_path(path)
        dirents = ['.', '..']
        if os.path.isdir(full_path):
            dirents.extend(os.listdir(full_path))
        return dirents

    @wrap_exeptions
    def readlink(self, path):
        pathname = os.readlink(self._full_path(path))
        if pathname.startswith("/"):
            # Path name is absolute, sanitize it.
            return os.path.relpath(pathname, self.root)
        else:
            return pathname

    @wrap_exeptions
    def mknod(self, path, mode, dev):
        return os.mknod(self._full_path(path), mode, dev)

    @wrap_exeptions
    def rmdir(self, path):
        full_path = self._full_path(path)
        return os.rmdir(full_path)

    @wrap_exeptions
    def mkdir(self, path, mode):
        return os.mkdir(self._full_path(path), mode)

    @wrap_exeptions
    def statfs(self, path):
        full_path = self._full_path(path)
        stv = os.statvfs(full_path)
        return dict((key, getattr(stv, key)) for key in ('f_bavail', 'f_bfree',
            'f_blocks', 'f_bsize', 'f_favail', 'f_ffree', 'f_files', 'f_flag',
            'f_frsize', 'f_namemax'))

    @wrap_exeptions
    def unlink(self, path):
        return os.unlink(self._full_path(path))

    @wrap_exeptions
    def symlink(self, name, target):
        return os.symlink(name, self._full_path(target))

    @wrap_exeptions
    def rename(self, old, new):
        return os.rename(self._full_path(old), self._full_path(new))

    @wrap_exeptions
    def link(self, target, name):
        return os.link(self._full_path(target), self._full_path(name))

    @wrap_exeptions
    def utimens(self, path, times):
        times = tuple(times)
        return os.utime(self._full_path(path), times)

    # File methods
    # ============

    @wrap_exeptions
    def open(self, path, flags):
        full_path = self._full_path(path)
        return os.open(full_path, flags)

    @wrap_exeptions
    def create(self, path, mode, fi=None):
        full_path = self._full_path(path)
        return os.open(full_path, os.O_WRONLY | os.O_CREAT, mode)

    @wrap_exeptions
    def read(self, path, length, offset, fh):
        os.lseek(fh, offset, os.SEEK_SET)
        data = os.read(fh, length)
        return data

    @wrap_exeptions
    def write(self, path, buf, offset, fh):
        buf = buf.data
        os.lseek(fh, offset, os.SEEK_SET)
        data = os.write(fh, buf)
        return data

    @wrap_exeptions
    def truncate(self, path, length, fh=None):
        full_path = self._full_path(path)
        with open(full_path, 'r+') as f:
            f.truncate(length)

    @wrap_exeptions
    def flush(self, path, fh):
        return os.fsync(fh)

    @wrap_exeptions
    def release(self, path, fh):
        return os.close(fh)

    @wrap_exeptions
    def fsync(self, path, fdatasync, fh):
        return self.flush(path, fh)


def main():

    # Get configuration
    server_name, server_port = SERVER_URL.split(':')
    server_port = int(server_port)
    root_dir = sys.argv[1]

    # Create server
    logger.info(f"Starting server on {server_name}:{server_port}")
    with SimpleXMLRPCServer(
        (server_name, server_port),
        requestHandler=SimpleXMLRPCRequestHandler,
        logRequests=False,
        allow_none=True,
    ) as server:

        server.register_introspection_functions()
        server.register_instance(Passthrough(root_dir))

        # Run the server's main loop
        logger.debug(f"Serving directory '{root_dir}'")
        server.serve_forever()

if __name__ == '__main__':
    main()
    